"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IsValidDateString = exports.IsDefined = exports.OptionalNotNullable = exports.IsNotNull = void 0;
const class_validator_1 = require("class-validator");
const dayjs_1 = __importDefault(require("dayjs"));
const customParseFormat_1 = __importDefault(require("dayjs/plugin/customParseFormat"));
const common_1 = require("@nestjs/common");
dayjs_1.default.extend(customParseFormat_1.default);
const IsNotNull = () => (0, common_1.applyDecorators)((0, class_validator_1.IsNotIn)([null], {
    message: '$property must not be null',
}));
exports.IsNotNull = IsNotNull;
const OptionalNotNullable = () => (0, common_1.applyDecorators)((0, class_validator_1.ValidateIf)((object, value) => value !== undefined), (0, exports.IsNotNull)());
exports.OptionalNotNullable = OptionalNotNullable;
const IsDefined = () => (0, common_1.applyDecorators)((0, class_validator_1.IsNotIn)([undefined], {
    message: '$property must be defined',
}));
exports.IsDefined = IsDefined;
const IsValidDateString = (format = 'YYYY-MM-DD[T]HH:mm:ss[Z]') => (0, class_validator_1.ValidateBy)({
    name: class_validator_1.IS_DATE_STRING,
    validator: {
        validate(value) {
            if (value instanceof Date) {
                return !Number.isNaN(value.getTime());
            }
            if (typeof value !== 'string') {
                return false;
            }
            return (0, dayjs_1.default)(value, format, true).isValid();
        },
        defaultMessage() {
            return `$property must be a valid date of the format ${format}`;
        },
    },
});
exports.IsValidDateString = IsValidDateString;
//# sourceMappingURL=validator.js.map