"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var AllExceptionsFilter_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AllExceptionsFilter = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const middlewares_1 = require("../middlewares");
const services_1 = require("../services");
const domain_1 = require("../domain");
let AllExceptionsFilter = AllExceptionsFilter_1 = class AllExceptionsFilter {
    constructor(httpAdapterHost, logger) {
        this.logger = logger;
        this.httpAdapter = httpAdapterHost.httpAdapter;
    }
    catch(exception, host) {
        const { response } = AllExceptionsFilter_1.getRequestAndResponse(host);
        const responseBody = this.formatErrors(host, exception);
        const { httpStatusCode } = responseBody;
        const errorAttributes = {
            ...responseBody,
            err: exception,
            'error.stack': exception.stack,
            'error.kind': exception.name,
            message: exception.message,
        };
        if (httpStatusCode >= 500) {
            this.logger.error(errorAttributes);
        }
        else {
            this.logger.warn(errorAttributes);
        }
        this.httpAdapter.reply(response, responseBody, httpStatusCode);
    }
    static getRequestAndResponse(host) {
        const context = host.switchToHttp();
        return {
            request: context.getRequest(),
            response: context.getResponse(),
        };
    }
    formatErrors(host, exception) {
        const { request } = AllExceptionsFilter_1.getRequestAndResponse(host);
        const httpStatus = exception instanceof common_1.HttpException
            ? exception.getStatus()
            : common_1.HttpStatus.INTERNAL_SERVER_ERROR;
        const message = httpStatus === common_1.HttpStatus.INTERNAL_SERVER_ERROR
            ? 'Internal Server Error'
            : exception.message;
        let errors = exception instanceof common_1.HttpException
            ? exception.getResponse()
            : message;
        if (typeof errors === 'object') {
            if (typeof errors.message === 'object') {
                errors = errors.message;
            }
        }
        else {
            errors = { message, code: domain_1.GenericErrorCodes.INTERNAL_SERVER_ERROR };
        }
        if (!Array.isArray(errors)) {
            errors = [errors];
        }
        errors = errors.map((err) => {
            const code = err.code ?? domain_1.GenericErrorCodes.INTERNAL_SERVER_ERROR;
            return {
                ...err,
                message: err.message ?? message,
                code,
            };
        });
        return {
            httpStatusCode: httpStatus,
            traceId: request.headers[middlewares_1.TRACE_ID],
            path: this.httpAdapter.getRequestUrl(request),
            errors,
        };
    }
};
AllExceptionsFilter = AllExceptionsFilter_1 = __decorate([
    (0, common_1.Catch)(),
    __metadata("design:paramtypes", [core_1.HttpAdapterHost,
        services_1.RootLogger])
], AllExceptionsFilter);
exports.AllExceptionsFilter = AllExceptionsFilter;
//# sourceMappingURL=exception.js.map