"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ValidationPipe_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateOptions = exports.ValidationPipe = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const lodash_1 = require("lodash");
const class_validator_1 = require("class-validator");
const domain_1 = require("../domain");
const VALIDATE_OPTIONS = 'validationOptions';
const constraintToApiErrorCode = {
    [class_validator_1.IS_EMAIL]: domain_1.GenericErrorCodes.INVALID_EMAIL,
    [class_validator_1.IS_INT]: domain_1.GenericErrorCodes.INVALID_NUMBER,
    [class_validator_1.IS_BOOLEAN]: domain_1.GenericErrorCodes.INVALID_BOOLEAN,
    [class_validator_1.MAX]: domain_1.GenericErrorCodes.VALUE_TOO_BIG,
    [class_validator_1.MIN]: domain_1.GenericErrorCodes.VALUE_TOO_SMALL,
    [class_validator_1.IS_URL]: domain_1.GenericErrorCodes.INVALID_URL,
    [class_validator_1.IS_IBAN]: domain_1.GenericErrorCodes.INVALID_IBAN,
    [class_validator_1.IS_BIC]: domain_1.GenericErrorCodes.INVALID_BIC,
    [class_validator_1.IS_NUMBER]: domain_1.GenericErrorCodes.INVALID_NUMBER,
    [class_validator_1.IS_DATE]: domain_1.GenericErrorCodes.INVALID_DATE,
    [class_validator_1.IS_DATE_STRING]: domain_1.GenericErrorCodes.INVALID_DATE,
    [class_validator_1.IS_NOT_EMPTY]: domain_1.GenericErrorCodes.VALUE_REQUIRED,
    [class_validator_1.ARRAY_NOT_EMPTY]: domain_1.GenericErrorCodes.VALUE_REQUIRED,
    [class_validator_1.MIN_LENGTH]: domain_1.GenericErrorCodes.INVALID_LENGTH,
    [class_validator_1.MAX_LENGTH]: domain_1.GenericErrorCodes.INVALID_LENGTH,
};
let ValidationPipe = ValidationPipe_1 = class ValidationPipe {
    constructor(reflector) {
        this.reflector = reflector;
    }
    async transform(value, metadata) {
        const { metatype } = metadata;
        if (!metatype) {
            return value;
        }
        if (metatype === Number) {
            return new common_1.ParseIntPipe({
                exceptionFactory: (err) => new common_1.BadRequestException({
                    message: err,
                    code: domain_1.GenericErrorCodes.INVALID_NUMBER,
                }),
            }).transform(value, metadata);
        }
        if (metatype === Boolean) {
            return new common_1.ParseBoolPipe({
                exceptionFactory: (err) => new common_1.BadRequestException({
                    message: err,
                    code: domain_1.GenericErrorCodes.INVALID_BOOLEAN,
                }),
            }).transform(value, metadata);
        }
        const options = this.reflector.get(VALIDATE_OPTIONS, metatype) ||
            {};
        const validationPipe = new common_1.ValidationPipe((0, lodash_1.merge)({
            whitelist: true,
            transform: true,
            transformOptions: {
                exposeUnsetFields: false,
            },
            forbidNonWhitelisted: true,
            exceptionFactory: (validationErrors = []) => new common_1.BadRequestException(ValidationPipe_1.transformValidationError(validationPipe, validationErrors)),
        }, options));
        return validationPipe.transform(value, metadata);
    }
    static transformValidationError(validationPipe, validationErrors) {
        return validationErrors
            .map((error) => validationPipe.mapChildrenToValidationErrors(error))
            .flat()
            .filter((item) => !!item.constraints)
            .map((item) => Object.entries(item.constraints || {}).map(([constraint, message]) => ({
            message,
            code: constraintToApiErrorCode[constraint] ??
                domain_1.GenericErrorCodes.INVALID_VALUE,
        })))
            .flat();
    }
};
ValidationPipe = ValidationPipe_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.Reflector])
], ValidationPipe);
exports.ValidationPipe = ValidationPipe;
/**
 * Allow setting custom ValidationOptions per domain model
 * @param validateOptions
 * usage:
 *
 * @ValidateOptions({ transform: false })
 * class CatDto {
 *
 * }
 */
const ValidateOptions = (validateOptions) => (0, common_1.SetMetadata)(VALIDATE_OPTIONS, validateOptions);
exports.ValidateOptions = ValidateOptions;
//# sourceMappingURL=validation.js.map